
/*ProgressChecker.java adapter to test Reflection time expense*/
/**checks time cost of creating new classes and calling those methods using 
 * Reflection**/

import java.lang.reflect.*;
import java.io.*;


public class PCClassCreator {
    
    public static void main(String[] args) {
  int numOfIter = 15000;
  long refTimeB = 50;
  long refTimeE = 550;
  long normTimeB = 60;
  long normTimeE = 660;
  
  //create new class
  create();
  
  //time using reflection
  try{
        //Compiles Add.java
        Process p = Runtime.getRuntime().exec("javac Add.java");
        p.waitFor();
        if(p.exitValue() == 0) {
            Class outputClassObject = Class.forName("Add");
            Class[] fpl = {String[].class};
            Method m = outputClassObject.getMethod("main", fpl);
   refTimeB = System.currentTimeMillis();
   for(int i = 0; i < numOfIter; i++)
    m.invoke(null, new Object[] {new String[] {"5", "6"} });
   refTimeE = System.currentTimeMillis();
        } else {
            InputStream errStream = p.getErrorStream();
            for(int j = errStream.available(); j > 0; j--)
              System.out.write(errStream.read());
        }
  } catch(Exception e) {}
  
  //time using direct method call
  normTimeB = System.currentTimeMillis();
  for(int i = 0; i < numOfIter; i++)
   add(new String[] {"5","6"});
  normTimeE = System.currentTimeMillis();
  
  /* NOTE 'mult' has same code as Multiply.main */
        double timeForREF = refTimeE-refTimeB;
        double timeForNORM = normTimeE-normTimeB;
        System.out.println("Time: " + timeForREF + 
    " ms for calling Add.main using Reflection.");
        System.out.println("Time: " + timeForNORM + 
    " ms for calling 'add' directly.");
        System.out.println("Average Time: " +"\n"+ 
    (timeForREF/numOfIter) + " ms for Reflection."+"\n"+
    (timeForNORM/numOfIter)+ " ms for Normal method call to add");
    }
 
 public static void create() {
  try {
            //creates a file called Multiply.java
            FileOutputStream fstream = new FileOutputStream("Multiply.java");
            PrintWriter out = new PrintWriter(fstream);
            //saves the following code to Multiply.java
            out.println(
                    "public class Add { \n" +                    
                    "public static void main(String[] args) { \n" +
                    "Integer x = new Integer(Integer.parseInt(args[0]));" +
     "Integer y = new Integer(Integer.parseInt(args[1]));" +
                    "int z = (x.intValue() + y.intValue());" +
                    "} \n" +
                    "} \n"
                    );
            //Purges the output stream
            out.flush();
  } catch (Exception e) {
  }
  }
 
 public static void add(String args[]) {
  Integer x = new Integer(Integer.parseInt(args[0]));
  Integer y = new Integer(Integer.parseInt(args[1]));
  int z = (x.intValue() + y.intValue());
 }
}
